﻿import { JsonSerializer } from '../../utilities/JsonSerializer';
import { DatasetProfileService } from '../../services/dataset-profile.service';
import { DatasetProfileModelAdmin } from '../../models/datasetProfileAdmin/DatasetProfileModelAdmin';
import { Page } from '../../models/datasetProfileAdmin/Page';
import { Component, OnInit } from '@angular/core';
import { FormGroup, FormControl } from '@angular/forms';
import { FormArray } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { DatasetProfileAdmin } from '../../services/datasetProfileAdmin/datasetProfileAfmin.service';
import { Section } from '../../models/datasetProfileAdmin/Section';
import { MatDialog } from '@angular/material';
import { DatasetProfilePreviewerComponent } from '../previewer/dataset-profile-previewer.component';

@Component({
	selector: 'app-form-component',
	templateUrl: './form.component.html',
	providers: [DatasetProfileAdmin, DatasetProfileService],
	styleUrls: ['./form.component.scss']
})

export class FormComponent implements OnInit {

	dataModel: DatasetProfileModelAdmin;
	form: FormGroup;
	private profileID: string;
	private cloneId: string;

	constructor(
		public datasetprofileAdmin: DatasetProfileAdmin,
		private datasetProfileService: DatasetProfileService,
		private route: ActivatedRoute,
		private router: Router,
		public dialog: MatDialog,
	) {
		this.profileID = route.snapshot.params['id'];
		this.cloneId = route.snapshot.params['cloneid'];
	}

	ngOnInit() {

		this.dataModel = JsonSerializer.fromJSONObject(new DatasetProfileModelAdmin(), DatasetProfileModelAdmin);
		this.form = this.dataModel.buildForm();
		if (this.profileID) {
			this.datasetProfileService.getDatasetProfileById(this.profileID).subscribe((data) => {
				this.dataModel = JsonSerializer.fromJSONObject(data, DatasetProfileModelAdmin);
				this.form = this.dataModel.buildForm();
			});
		} else if (this.cloneId) {
			this.datasetprofileAdmin.clone(this.cloneId).subscribe((data) => {
				this.dataModel = JsonSerializer.fromJSONObject(data, DatasetProfileModelAdmin);
				this.form = this.dataModel.buildForm();
			});
		} else {
			this.addSection();
			this.addPage();
		}
	}

	onIsMultiplicityEnabledChange(isMultiplicityEnabled: boolean) {
		if (!isMultiplicityEnabled) {
			(<FormControl>this.form.get('multiplicity').get('min')).setValue(0);
			(<FormControl>this.form.get('multiplicity').get('max')).setValue(0);
		}
	}

	addSection() {
		const section: Section = new Section();
		this.dataModel.sections.push(section);
		(<FormArray>this.form.get('sections')).push(section.buildForm());
	}

	addPage() {
		const page: Page = new Page(this.dataModel.pages.length);
		this.dataModel.pages.push(page);
		(<FormArray>this.form.get('pages')).push(page.buildForm());
	}

	DeleteSection(index) {
		this.dataModel.sections.splice(index, 1);
		(<FormArray>this.form.get('sections')).removeAt(index);
	}

	DeletePage(index) {
		this.dataModel.pages.splice(index, 1);
		(<FormArray>this.form.get('pages')).removeAt(index);
	}

	createForm(data) {
		return this.datasetprofileAdmin.createForm(data);
	}
	updateForm(id, data) {
		return this.datasetprofileAdmin.updateForm(id, data);
	}

	onSubmit() {
		const data = this.form.value;

		if (this.profileID) {
			this.updateForm(this.profileID, data).subscribe(() => {
				this.router.navigate(['/dataset-profile']);
			});
		} else {
			this.createForm(data).subscribe(() => {
				this.router.navigate(['/dataset-profile']);
			});
		}
	}

	preview() {
		const dialogRef = this.dialog.open(DatasetProfilePreviewerComponent, {
			height: '355px',
			width: '700px',
			data: {
				model: this.dataModel
			}
		});
	}

}
